@@ -206,16 +206,67 @@ class cTestPanel(gmBusinessDBObject.cBusinessDBObject):
 		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 		return True
 	#--------------------------------------------------------
+	def get_test_types_for_results(self, pk_patient, order_by=None, unique_meta_types=False):
+		"""Retrieve data about test types on this panel (for which this patient has results)."""
+
+		if order_by is None:
+			order_by = u''
+		else:
+			order_by = u'ORDER BY %s' % order_by
+
+		if unique_meta_types:
+			cmd = u"""
+				SELECT * FROM clin.v_test_types c_vtt
+				WHERE c_vtt.pk_test_type IN (
+						SELECT DISTINCT ON (c_vtr1.pk_meta_test_type) c_vtr1.pk_test_type
+						FROM clin.v_test_results c_vtr1
+						WHERE
+							c_vtr1.pk_test_type IN %%(pks)s
+								AND
+							c_vtr1.pk_patient = %%(pat)s
+								AND
+							c_vtr1.pk_meta_test_type IS NOT NULL
+					UNION ALL
+						SELECT DISTINCT ON (c_vtr2.pk_test_type) c_vtr2.pk_test_type
+						FROM clin.v_test_results c_vtr2
+						WHERE
+							c_vtr2.pk_test_type IN %%(pks)s
+								AND
+							c_vtr2.pk_patient = %%(pat)s
+								AND
+							c_vtr2.pk_meta_test_type IS NULL
+				)
+				%s""" % order_by
+		else:
+			cmd = u"""
+				SELECT * FROM clin.v_test_types c_vtt
+				WHERE c_vtt.pk_test_type IN (
+					SELECT DISTINCT ON (c_vtr.pk_test_type) c_vtr.pk_test_type
+					FROM clin.v_test_results c_vtr
+					WHERE
+						c_vtr.pk_test_type IN %%(pks)s
+								AND
+						c_vtr.pk_patient = %%(pat)s
+				)
+				%s""" % order_by
+
+		args = {
+			'pat': pk_patient,
+			'pks': tuple(self._payload[self._idx['pk_test_types']])
+		}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return [ cMeasurementType(row = {'pk_field': 'pk_test_type', 'idx': idx, 'data': r}) for r in rows ]
+
+	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
 	def _get_test_types(self):
 		if self._payload[self._idx['pk_test_types']] is None:
 			return None
-
 		rows, idx = gmPG2.run_ro_queries (
 			queries = [{
-			'cmd': _SQL_get_test_types % u'pk_test_type IN %(pks)s ORDER BY unified_abbrev',
-			'args': {'pks': tuple(self._payload[self._idx['pk_test_types']])}
+				'cmd': _SQL_get_test_types % u'pk_test_type IN %(pks)s ORDER BY unified_abbrev',
+				'args': {'pks': tuple(self._payload[self._idx['pk_test_types']])}
 			}],
 			get_col_idx = True
 		)
@@ -329,18 +380,19 @@ class cMetaTestType(gmBusinessDBObject.cBusinessDBObject):
 			self._payload[self._idx['name']]
 		)
 		if self._payload[self._idx['loinc']] is not None:
-			txt += u' LOINC: %s (%s)\n' % self._payload[self._idx['loinc']]
-		if self._payload[self._idx['loinc']] is not None:
+			txt += u' LOINC: %s\n' % self._payload[self._idx['loinc']]
+		if self._payload[self._idx['comment']] is not None:
 			txt += _(' Comment: %s\n') % self._payload[self._idx['comment']]
 		if with_tests:
 			ttypes = self.included_test_types
 			if len(ttypes) > 0:
 				txt += _(' Aggregates the following test types:\n')
 			for ttype in ttypes:
-				txt += u'  %s (%s)%s%s      [#%s]\n' % (
-					ttype['abbrev'],
+				txt += u'  - %s (%s)%s%s%s      [#%s]\n' % (
 					ttype['name'],
+					ttype['abbrev'],
 					gmTools.coalesce(ttype['conversion_unit'], u'', ', %s'),
+					gmTools.coalesce(ttype['name_org'], u'', u' (%s)'),
 					gmTools.coalesce(ttype['loinc'], u'', u', LOINC: %s'),
 					ttype['pk_test_type']
 				)
@@ -349,7 +401,7 @@ class cMetaTestType(gmBusinessDBObject.cBusinessDBObject):
 			most_recent = self.get_most_recent_result(patient = patient)
 			if most_recent is not None:
 				txt += _(' Most recent (%s): %s%s%s') % (
-					most_recent['clin_when'].strftime('%Y %b %d'),
+					gmDateTime.pydt_strftime(most_recent['clin_when'], '%Y %b %d'),
 					most_recent['unified_val'],
 					gmTools.coalesce(most_recent['val_unit'], u'', u' %s'),
 					gmTools.coalesce(most_recent['abnormality_indicator'], u'', u' (%s)')
@@ -358,7 +410,7 @@ class cMetaTestType(gmBusinessDBObject.cBusinessDBObject):
 			if oldest is not None:
 				txt += u'\n'
 				txt += _(' Oldest (%s): %s%s%s') % (
-					oldest['clin_when'].strftime('%Y %b %d'),
+					gmDateTime.pydt_strftime(oldest['clin_when'], '%Y %b %d'),
 					oldest['unified_val'],
 					gmTools.coalesce(oldest['val_unit'], u'', u' %s'),
 					gmTools.coalesce(oldest['abnormality_indicator'], u'', u' (%s)')
@@ -721,7 +773,7 @@ LIMIT 1"""
 			result = self.get_most_recent_results(patient = patient, no_of_results = 1)
 			if result is not None:
 				tt += _(' Most recent (%s): %s%s%s') % (
-					result['clin_when'].strftime('%Y-%m-%d'),
+					gmDateTime.pydt_strftime(result['clin_when'], '%Y %b %d'),
 					result['unified_val'],
 					gmTools.coalesce(result['val_unit'], u'', u' %s'),
 					gmTools.coalesce(result['abnormality_indicator'], u'', u' (%s)')
@@ -730,7 +782,7 @@ LIMIT 1"""
 			if result is not None:
 				tt += u'\n'
 				tt += _(' Oldest (%s): %s%s%s') % (
-					result['clin_when'].strftime('%Y-%m-%d'),
+					gmDateTime.pydt_strftime(result['clin_when'], '%Y %b %d'),
 					result['unified_val'],
 					gmTools.coalesce(result['val_unit'], u'', u' %s'),
 					gmTools.coalesce(result['abnormality_indicator'], u'', u' (%s)')
@@ -904,36 +956,42 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 		'pk_request'
 	]
 	#--------------------------------------------------------
+	def format_concisely(self, date_format='%Y %b %d', with_notes=True):
+		range_info = gmTools.coalesce (
+			self.formatted_clinical_range,
+			self.formatted_normal_range
+		)
+		review = gmTools.bool2subst (
+			self._payload[self._idx['reviewed']],
+			u'',
+			u' ' + gmTools.u_writing_hand,
+			u' ' + gmTools.u_writing_hand
+		)
+		txt = u'%s %s: %s%s%s%s%s' % (
+			gmDateTime.pydt_strftime (
+				self._payload[self._idx['clin_when']],
+				date_format
+			),
+			self._payload[self._idx['name_tt']],
+			self._payload[self._idx['unified_val']],
+			gmTools.coalesce(self._payload[self._idx['val_unit']], u'', u' %s'),
+			gmTools.coalesce(self._payload[self._idx['abnormality_indicator']], u'', u' %s'),
+			gmTools.coalesce(range_info, u'', u' (%s)'),
+			review
+		)
+		if with_notes:
+			txt += u'\n'
+			if self._payload[self._idx['note_test_org']] is not None:
+				txt += u' ' + _(u'Lab comment: %s\n') % _(u'\n Lab comment: ').join(self._payload[self._idx['note_test_org']].split(u'\n'))
+			if self._payload[self._idx['comment']] is not None:
+				txt += u' ' + _(u'Praxis comment: %s\n') % _(u'\n Praxis comment: ').join(self._payload[self._idx['comment']].split(u'\n'))
+
+		return txt.strip(u'\n')
+	#--------------------------------------------------------
 	def format(self, with_review=True, with_evaluation=True, with_ranges=True, with_episode=True, with_type_details=True, date_format='%Y %b %d %H:%M'):
 
 		# FIXME: add battery, request details
 
-		has_normal_min_or_max = (
-			self._payload[self._idx['val_normal_min']] is not None
-		) or (
-			self._payload[self._idx['val_normal_max']] is not None
-		)
-		if has_normal_min_or_max:
-			normal_min_max = u'%s - %s' % (
-				gmTools.coalesce(self._payload[self._idx['val_normal_min']], u'?'),
-				gmTools.coalesce(self._payload[self._idx['val_normal_max']], u'?')
-			)
-		else:
-			normal_min_max = u''
-
-		has_clinical_min_or_max = (
-			self._payload[self._idx['val_target_min']] is not None
-		) or (
-			self._payload[self._idx['val_target_max']] is not None
-		)
-		if has_clinical_min_or_max:
-			clinical_min_max = u'%s - %s' % (
-				gmTools.coalesce(self._payload[self._idx['val_target_min']], u'?'),
-				gmTools.coalesce(self._payload[self._idx['val_target_max']], u'?')
-			)
-		else:
-			clinical_min_max = u''
-
 		# header
 		tt = _(u'Result from %s             \n') % gmDateTime.pydt_strftime (
 			self._payload[self._idx['clin_when']],
@@ -952,13 +1010,6 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 			'ind': gmTools.coalesce(self._payload[self._idx['abnormality_indicator']], u'', u' (%s)'),
 			'pk_result': self._payload[self._idx['pk_test_result']]
 		})
-		tmp = (u'%s%s' % (
-			gmTools.coalesce(self._payload[self._idx['name_test_org']], u''),
-			gmTools.coalesce(self._payload[self._idx['contact_test_org']], u'', u' (%s)'),
-		)).strip()
-		if tmp != u'':
-			tt += u' ' + _(u'Source: %s\n') % tmp
-		tt += u'\n'
 
 		if with_evaluation:
 			norm_eval = None
@@ -987,7 +1038,7 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 						else:
 							norm_eval = _(u'%.0f times the normal upper limit') % x_times
 				if norm_eval is not None:
-					tt += u'  (%s)\n' % norm_eval
+					tt += u'  = %s\n' % norm_eval
 	#			#-------------------------------------
 	#			# this idea was shot down on the list
 	#			#-------------------------------------
@@ -1038,7 +1089,7 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 						else:
 							norm_eval = _(u'%.0f times the target upper limit') % x_times
 				if norm_eval is not None:
-					tt += u' (%s)\n' % norm_eval
+					tt += u' = %s\n' % norm_eval
 	#			#-------------------------------------
 	#			# this idea was shot down on the list
 	#			#-------------------------------------
@@ -1064,39 +1115,24 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 	#						tt += u'  (%s)\n' % _(u'deviates by %.0f times of the target range') % times_deviation
 	#			#-------------------------------------
 
+		tmp = (u'%s%s' % (
+			gmTools.coalesce(self._payload[self._idx['name_test_org']], u''),
+			gmTools.coalesce(self._payload[self._idx['contact_test_org']], u'', u' (%s)'),
+		)).strip()
+		if tmp != u'':
+			tt += u' ' + _(u'Source: %s\n') % tmp
+		tt += u'\n'
+		if self._payload[self._idx['note_test_org']] is not None:
+			tt += u' ' + _(u'Lab comment: %s\n') % _(u'\n Lab comment: ').join(self._payload[self._idx['note_test_org']].split(u'\n'))
+		if self._payload[self._idx['comment']] is not None:
+			tt += u' ' + _(u'Praxis comment: %s\n') % _(u'\n Praxis comment: ').join(self._payload[self._idx['comment']].split(u'\n'))
+
 		if with_ranges:
-			tt += u' ' + _(u'Standard normal range: %(norm_min_max)s%(norm_range)s  \n') % ({
-				'norm_min_max': normal_min_max,
-				'norm_range': gmTools.coalesce (
-					self._payload[self._idx['val_normal_range']],
-					u'',
-					gmTools.bool2subst (
-						has_normal_min_or_max,
-						u' / %s',
-						u'%s'
-					)
-				)
-			})
-			if self._payload[self._idx['norm_ref_group']] is not None:
-				tt += u' ' + _(u'Reference group: %s\n') % self._payload[self._idx['norm_ref_group']]
-			tt += u' ' + _(u'Clinical target range: %(clin_min_max)s%(clin_range)s  \n') % ({
-				'clin_min_max': clinical_min_max,
-				'clin_range': gmTools.coalesce (
-					self._payload[self._idx['val_target_range']],
-					u'',
-					gmTools.bool2subst (
-						has_clinical_min_or_max,
-						u' / %s',
-						u'%s'
-					)
-				)
-			})
+			tt += gmTools.coalesce(self.formatted_normal_range, u'', u' ' + _('Standard normal range: %s\n'))
+			tt += gmTools.coalesce(self.formatted_clinical_range, u'', u' ' + _('Clinical target range: %s\n'))
+			tt += gmTools.coalesce(self._payload[self._idx['norm_ref_group']], u'', u' ' + _('Reference group: %s\n'))
 
 		# metadata
-		if self._payload[self._idx['comment']] is not None:
-			tt += u' ' + _(u'Doc: %s\n') % _(u'\n Doc: ').join(self._payload[self._idx['comment']].split(u'\n'))
-		if self._payload[self._idx['note_test_org']] is not None:
-			tt += u' ' + _(u'Lab: %s\n') % _(u'\n Lab: ').join(self._payload[self._idx['note_test_org']].split(u'\n'))
 		if with_episode:
 			tt += u' ' + _(u'Episode: %s\n') % self._payload[self._idx['episode']]
 			if self._payload[self._idx['health_issue']] is not None:
@@ -1155,7 +1191,8 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 		# type
 		if with_type_details:
 			tt += _(u'Test type details:\n')
-			tt += u' ' + _(u'Grouped under "%(name_meta)s" (%(abbrev_meta)s)  [#%(pk_u_type)s]\n') % ({
+			tt += u' ' + _(u'Aggregated (%(sum)s) under "%(name_meta)s" (%(abbrev_meta)s)  [#%(pk_u_type)s]\n') % ({
+				'sum': gmTools.u_sum,
 				'name_meta': gmTools.coalesce(self._payload[self._idx['name_meta']], u''),
 				'abbrev_meta': gmTools.coalesce(self._payload[self._idx['abbrev_meta']], u''),
 				'pk_u_type': self._payload[self._idx['pk_meta_test_type']]
@@ -1175,6 +1212,114 @@ class cTestResult(gmBusinessDBObject.cBusinessDBObject):
 
 		return tt
 	#--------------------------------------------------------
+	def _get_has_normal_min_or_max(self):
+		return (
+			self._payload[self._idx['val_normal_min']] is not None
+		) or (
+			self._payload[self._idx['val_normal_max']] is not None
+		)
+
+	has_normal_min_or_max = property(_get_has_normal_min_or_max, lambda x:x)
+	#--------------------------------------------------------
+	def _get_normal_min_max(self):
+		has_range_info = (
+			self._payload[self._idx['val_normal_min']] is not None
+		) or (
+			self._payload[self._idx['val_normal_max']] is not None
+		)
+		if has_range_info is False:
+			return None
+
+		return u'%s - %s' % (
+			gmTools.coalesce(self._payload[self._idx['val_normal_min']], u'?'),
+			gmTools.coalesce(self._payload[self._idx['val_normal_max']], u'?')
+		)
+
+	normal_min_max = property(_get_normal_min_max, lambda x:x)
+	#--------------------------------------------------------
+	def _get_formatted_normal_range(self):
+		has_numerical_range = (
+			self._payload[self._idx['val_normal_min']] is not None
+		) or (
+			self._payload[self._idx['val_normal_max']] is not None
+		)
+		if has_numerical_range:
+			numerical_range = u'%s - %s' % (
+				gmTools.coalesce(self._payload[self._idx['val_normal_min']], u'?'),
+				gmTools.coalesce(self._payload[self._idx['val_normal_max']], u'?')
+			)
+		else:
+			numerical_range = u''
+		textual_range = gmTools.coalesce (
+			self._payload[self._idx['val_normal_range']],
+			u'',
+			gmTools.bool2subst (
+				has_numerical_range,
+				u' / %s',
+				u'%s'
+			)
+		)
+		range_info = u'%s%s' % (numerical_range, textual_range)
+		if range_info == u'':
+			return None
+		return range_info
+
+	formatted_normal_range = property(_get_formatted_normal_range, lambda x:x)
+	#--------------------------------------------------------
+	def _get_has_clinical_min_or_max(self):
+		return (
+			self._payload[self._idx['val_target_min']] is not None
+		) or (
+			self._payload[self._idx['val_target_max']] is not None
+		)
+
+	has_clinical_min_or_max = property(_get_has_clinical_min_or_max, lambda x:x)
+	#--------------------------------------------------------
+	def _get_clinical_min_max(self):
+		has_range_info = (
+			self._payload[self._idx['val_target_min']] is not None
+		) or (
+			self._payload[self._idx['val_target_max']] is not None
+		)
+		if has_range_info is False:
+			return None
+
+		return u'%s - %s' % (
+			gmTools.coalesce(self._payload[self._idx['val_target_min']], u'?'),
+			gmTools.coalesce(self._payload[self._idx['val_target_max']], u'?')
+		)
+
+	clinical_min_max = property(_get_clinical_min_max, lambda x:x)
+	#--------------------------------------------------------
+	def _get_formatted_clinical_range(self):
+		has_numerical_range = (
+			self._payload[self._idx['val_target_min']] is not None
+		) or (
+			self._payload[self._idx['val_target_max']] is not None
+		)
+		if has_numerical_range:
+			numerical_range = u'%s - %s' % (
+				gmTools.coalesce(self._payload[self._idx['val_target_min']], u'?'),
+				gmTools.coalesce(self._payload[self._idx['val_target_max']], u'?')
+			)
+		else:
+			numerical_range = u''
+		textual_range = gmTools.coalesce (
+			self._payload[self._idx['val_target_range']],
+			u'',
+			gmTools.bool2subst (
+				has_numerical_range,
+				u' / %s',
+				u'%s'
+			)
+		)
+		range_info = u'%s%s' % (numerical_range, textual_range)
+		if range_info == u'':
+			return None
+		return range_info
+
+	formatted_clinical_range = property(_get_formatted_clinical_range, lambda x:x)
+	#--------------------------------------------------------
 	def _get_temporally_closest_normal_range(self):
 		"""Returns the closest test result which does have normal range information."""
 		if self._payload[self._idx['val_normal_min']] is not None:
